import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

import 'models.dart';

typedef CarillonMapBoxCreatedCallback = void Function(
    CarillonMapBoxController controller);

class CarillonMapBox extends StatefulWidget {
  CarillonMapBox(
      {Key? key, required this.onCarillonMapBoxCreated, required this.options})
      : super(key: key) {
    _channel.setMethodCallHandler(_handleMethod);
  }

  final CarillonMapBoxCreatedCallback onCarillonMapBoxCreated;
  final Map<String, dynamic> options;
  static const MethodChannel _channel = MethodChannel('carillon_map_box');
  static Function _startCallback = () {};
  static Function _endCallback = () {};
  static Function _popupButtonCallback = () {};

  Future<dynamic> _handleMethod(MethodCall call) async {
    if (call.method == "startCallback") {
      _startCallback();
    } else if (call.method == "endCallback") {
      _endCallback();
    } else if (call.method == "popupButtonCallback") {
      _popupButtonCallback();
    }
    /*switch (call.method) {
      case "message":
        debugPrint(call.arguments);
        return new Future.value("");
    }*/
  }

  static void startNavigation(Map<String, dynamic> options,
      Function startCallback, Function endCallback) async {
    _startCallback = startCallback;
    _endCallback = endCallback;
    _channel.invokeMethod<Map<String, dynamic>>('startNavigation', options);
  }

  static void sendAlert(Function popupButtonCallback) async {
    _popupButtonCallback = popupButtonCallback;
    _channel.invokeMethod('sendAlert');
  }

  @override
  State<StatefulWidget> createState() => _CarillonMapBoxState();
}

class _CarillonMapBoxState extends State<CarillonMapBox> {
  @override
  Widget build(BuildContext context) {
    final Map<String, dynamic> creationParams = <String, dynamic>{};
    if (defaultTargetPlatform == TargetPlatform.android) {
      return PlatformViewLink(
        viewType: 'plugins/carillon_map_box',
        surfaceFactory:
            (BuildContext context, PlatformViewController controller) {
          return AndroidViewSurface(
            controller: controller as AndroidViewController,
            gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
            hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          );
        },
        onCreatePlatformView: (PlatformViewCreationParams params) {
          return PlatformViewsService.initSurfaceAndroidView(
            id: params.id,
            viewType: 'plugins/carillon_map_box',
            layoutDirection: TextDirection.ltr,
            creationParams: widget.options,
            creationParamsCodec: StandardMessageCodec(),
            onFocus: () {
              params.onFocusChanged(true);
            },
          )
            ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
            ..create();
        },
      );
    }
    return const Text('iOS platform version is not implemented yet.');
  }

  void _onPlatformViewCreated(int id) {
    if (widget.onCarillonMapBoxCreated == null) {
      return;
    }
    widget.onCarillonMapBoxCreated(CarillonMapBoxController._(id));
  }
}

class CarillonMapBoxController {
  CarillonMapBoxController._(int id)
      : _channel = MethodChannel('plugins/carillon_map_box_$id');

  final MethodChannel _channel;

  MethodChannel getChannel() {
    return _channel;
  }
}
